using System;
using System.Windows.Forms;
using ChartDirector;


namespace SurfaceChartRotation
{
    public partial class FrmSurfaceChartRotation : Form
    {
        // The view angles
        private double rotationAngle = 45;
        private double elevationAngle = 30;

        public FrmSurfaceChartRotation()
        {
            InitializeComponent();
        }

        private void FrmSurfaceChartRotation_Load(object sender, EventArgs e)
        {
            // Display the chart
            winChartViewer1.updateViewPort(true, false);
        }

        //
        // The ViewPortChanged event handler
        //
        private void winChartViewer1_ViewPortChanged(object sender, WinViewPortEventArgs e)
        {
            // Update the chart if necessary
            if (e.NeedUpdateChart)
                drawChart(winChartViewer1);
        }

        public void drawChart(WinChartViewer viewer)
        {
            // The x and y coordinates of the grid
            double[] dataX = {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                10};
            double[] dataY = {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                10};

            // The values at the grid points. In this example, we will compute the values using the
            // formula z = x * sin(y) + y * sin(x).
            double[] dataZ = new double[dataX.Length * dataY.Length];
            for (int yIndex = 0; yIndex < dataY.Length; ++yIndex)
            {
                double y = dataY[yIndex];
                for (int xIndex = 0; xIndex < dataX.Length; ++xIndex)
                {
                    double x = dataX[xIndex];
                    dataZ[yIndex * dataX.Length + xIndex] = x * Math.Sin(y) + y * Math.Sin(x);
                }
            }

            // Create a SurfaceChart object of size 720 x 600 pixels
            SurfaceChart c = new SurfaceChart(720, 600);

            // Set the center of the plot region at (330, 290), and set width x depth x height to
            // 360 x 360 x 270 pixels
            c.setPlotRegion(330, 290, 360, 360, 270);

            // Set the data to use to plot the chart
            c.setData(dataX, dataY, dataZ);

            // Spline interpolate data to a 80 x 80 grid for a smooth surface
            c.setInterpolation(80, 80);

            // Set the view angles
            c.setViewAngle(elevationAngle, rotationAngle);

            // Check if draw frame only during rotation
            if (isDragging && DrawFrameOnRotate.Checked)
                c.setShadingMode(Chart.RectangularFrame);

            // Add a color axis (the legend) in which the left center is anchored at (660, 270). Set
            // the length to 200 pixels and the labels on the right side.
            c.setColorAxis(650, 270, Chart.Left, 200, Chart.Right);

            // Set the x, y and z axis titles using 10 points Arial Bold font
            c.xAxis().setTitle("X", "Arial Bold", 15);
            c.yAxis().setTitle("Y", "Arial Bold", 15);

            // Set axis label font
            c.xAxis().setLabelStyle("Arial", 10);
            c.yAxis().setLabelStyle("Arial", 10);
            c.zAxis().setLabelStyle("Arial", 10);
            c.colorAxis().setLabelStyle("Arial", 10);

            // Output the chart
            viewer.Chart = c;
        }

        // keep track of the last mouse position to compute mouse movement
        private int lastMouseX = -1;
        private int lastMouseY = -1;
        private bool isDragging = false;

        private void winChartViewer1_MouseDown(object sender, MouseEventArgs e)
        {
            if (0 != (e.Button & MouseButtons.Left))
            {
                // Start Drag
                isDragging = true;
                lastMouseX = e.X;
                lastMouseY = e.Y;
                (sender as WinChartViewer).updateViewPort(true, false);
            }
        }

        private void winChartViewer1_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                // The chart is configured to rotate by 90 degrees when the mouse moves from 
                // left to right, which is the plot region width (360 pixels). Similarly, the
                // elevation changes by 90 degrees when the mouse moves from top to buttom,
                // which is the plot region height (270 pixels).
                rotationAngle += (lastMouseX - e.X) * 90.0 / 360;
                elevationAngle += (e.Y - lastMouseY) * 90.0 / 270;
                lastMouseX = e.X;
                lastMouseY = e.Y;
                (sender as WinChartViewer).updateViewPort(true, false);
            }
        }

        private void winChartViewer1_MouseUp(object sender, MouseEventArgs e)
        {
            if (0 != (e.Button & MouseButtons.Left))
            {
                // End Drag
                isDragging = false;
                (sender as WinChartViewer).updateViewPort(true, false);
            }
        }
    }
}